
/*
 * Copyright (C) Vadim Zhestkov
 * Copyright (C) F5, Inc.
 */


NJS_DEF_STRING(unknown, "\xFF\xFF", 0, NJS_TOKEN_ILLEGAL)

NJS_DEF_SYMBOL(asyncIterator, "Symbol.asyncIterator")
NJS_DEF_SYMBOL(hasInstance, "Symbol.hasInstance")
NJS_DEF_SYMBOL(isConcatSpreadable, "Symbol.isConcatSpreadable")
NJS_DEF_SYMBOL(iterator, "Symbol.iterator")
NJS_DEF_SYMBOL(match, "Symbol.match")
NJS_DEF_SYMBOL(matchAll, "Symbol.matchAll")
NJS_DEF_SYMBOL(replace, "Symbol.replace")
NJS_DEF_SYMBOL(search, "Symbol.search")
NJS_DEF_SYMBOL(species, "Symbol.species")
NJS_DEF_SYMBOL(split, "Symbol.split")
NJS_DEF_SYMBOL(toPrimitive, "Symbol.toPrimitive")
NJS_DEF_SYMBOL(toStringTag, "Symbol.toStringTag")
NJS_DEF_SYMBOL(unscopables, "Symbol.unscopables")

/* Keywords. */

#define NJS_KWD (NJS_KEYWORD_TYPE_KEYWORD)
#define NJS_KWD_RESERVED (NJS_KEYWORD_TYPE_RESERVED | NJS_KEYWORD_TYPE_KEYWORD)

NJS_DEF_STRING(arguments, "arguments", NJS_KWD, NJS_TOKEN_ARGUMENTS)
NJS_DEF_STRING(async, "async", NJS_KWD, NJS_TOKEN_ASYNC)
NJS_DEF_STRING(await, "await", NJS_KWD_RESERVED, NJS_TOKEN_AWAIT)
NJS_DEF_STRING(break, "break", NJS_KWD_RESERVED, NJS_TOKEN_BREAK)
NJS_DEF_STRING(case, "case", NJS_KWD_RESERVED, NJS_TOKEN_CASE)
NJS_DEF_STRING(catch, "catch", NJS_KWD_RESERVED, NJS_TOKEN_CATCH)
NJS_DEF_STRING(class, "class", NJS_KWD_RESERVED, NJS_TOKEN_CLASS)
NJS_DEF_STRING(const, "const", NJS_KWD_RESERVED, NJS_TOKEN_CONST)
NJS_DEF_STRING(continue, "continue", NJS_KWD_RESERVED, NJS_TOKEN_CONTINUE)
NJS_DEF_STRING(debugger, "debugger", NJS_KWD_RESERVED, NJS_TOKEN_DEBUGGER)
NJS_DEF_STRING(default, "default", NJS_KWD_RESERVED, NJS_TOKEN_DEFAULT)
NJS_DEF_STRING(delete, "delete", NJS_KWD_RESERVED, NJS_TOKEN_DELETE)
NJS_DEF_STRING(do, "do", NJS_KWD_RESERVED, NJS_TOKEN_DO)
NJS_DEF_STRING(else, "else", NJS_KWD_RESERVED, NJS_TOKEN_ELSE)
NJS_DEF_STRING(enum, "enum", NJS_KWD_RESERVED, NJS_TOKEN_ENUM)
NJS_DEF_STRING(eval, "eval", NJS_KWD, NJS_TOKEN_EVAL)
NJS_DEF_STRING(export, "export", NJS_KWD_RESERVED, NJS_TOKEN_EXPORT)
NJS_DEF_STRING(extends, "extends", NJS_KWD_RESERVED, NJS_TOKEN_EXTENDS)
NJS_DEF_STRING(false, "false", NJS_KWD_RESERVED, NJS_TOKEN_FALSE)
NJS_DEF_STRING(finally, "finally", NJS_KWD_RESERVED, NJS_TOKEN_FINALLY)
NJS_DEF_STRING(for, "for", NJS_KWD_RESERVED, NJS_TOKEN_FOR)
NJS_DEF_STRING(from, "from", NJS_KWD, NJS_TOKEN_FROM)
NJS_DEF_STRING(function, "function", NJS_KWD_RESERVED, NJS_TOKEN_FUNCTION)
NJS_DEF_STRING(if, "if", NJS_KWD_RESERVED, NJS_TOKEN_IF)
NJS_DEF_STRING(implements, "implements", NJS_KWD_RESERVED, NJS_TOKEN_IMPLEMENTS)
NJS_DEF_STRING(import, "import", NJS_KWD_RESERVED, NJS_TOKEN_IMPORT)
NJS_DEF_STRING(in, "in", NJS_KWD_RESERVED, NJS_TOKEN_IN)
NJS_DEF_STRING(instanceof, "instanceof", NJS_KWD_RESERVED, NJS_TOKEN_INSTANCEOF)
NJS_DEF_STRING(interface, "interface", NJS_KWD_RESERVED, NJS_TOKEN_INTERFACE)
NJS_DEF_STRING(let, "let", NJS_KWD_RESERVED, NJS_TOKEN_LET)
NJS_DEF_STRING(meta, "meta", NJS_KWD, NJS_TOKEN_META)
NJS_DEF_STRING(new, "new", NJS_KWD_RESERVED, NJS_TOKEN_NEW)
NJS_DEF_STRING(null, "null", NJS_KWD_RESERVED, NJS_TOKEN_NULL)
NJS_DEF_STRING(of, "of", NJS_KWD, NJS_TOKEN_OF)
NJS_DEF_STRING(package, "package", NJS_KWD_RESERVED, NJS_TOKEN_PACKAGE)
NJS_DEF_STRING(private, "private", NJS_KWD_RESERVED, NJS_TOKEN_PRIVATE)
NJS_DEF_STRING(protected, "protected", NJS_KWD_RESERVED, NJS_TOKEN_PROTECTED)
NJS_DEF_STRING(public, "public", NJS_KWD_RESERVED, NJS_TOKEN_PUBLIC)
NJS_DEF_STRING(return, "return", NJS_KWD_RESERVED, NJS_TOKEN_RETURN)
NJS_DEF_STRING(static, "static", NJS_KWD_RESERVED, NJS_TOKEN_STATIC)
NJS_DEF_STRING(super, "super", NJS_KWD_RESERVED, NJS_TOKEN_SUPER)
NJS_DEF_STRING(switch, "switch", NJS_KWD_RESERVED, NJS_TOKEN_SWITCH)
NJS_DEF_STRING(target, "target", NJS_KWD, NJS_TOKEN_TARGET)
NJS_DEF_STRING(this, "this", NJS_KWD_RESERVED, NJS_TOKEN_THIS)
NJS_DEF_STRING(throw, "throw", NJS_KWD_RESERVED, NJS_TOKEN_THROW)
NJS_DEF_STRING(true, "true", NJS_KWD_RESERVED, NJS_TOKEN_TRUE)
NJS_DEF_STRING(try, "try", NJS_KWD_RESERVED, NJS_TOKEN_TRY)
NJS_DEF_STRING(typeof, "typeof", NJS_KWD_RESERVED, NJS_TOKEN_TYPEOF)
NJS_DEF_STRING(undefined, "undefined",  NJS_KWD, NJS_TOKEN_UNDEFINED)
NJS_DEF_STRING(var, "var", NJS_KWD_RESERVED, NJS_TOKEN_VAR)
NJS_DEF_STRING(void, "void", NJS_KWD_RESERVED, NJS_TOKEN_VOID)
NJS_DEF_STRING(while, "while", NJS_KWD_RESERVED, NJS_TOKEN_WHILE)
NJS_DEF_STRING(with, "with", NJS_KWD_RESERVED, NJS_TOKEN_WITH)
NJS_DEF_STRING(yield, "yield", NJS_KWD_RESERVED, NJS_TOKEN_YIELD)

/* Other predefined strings. */

NJS_DEF_STRING(spec_EMPTY_REGEXP, "(?:)", 0, 0)

NJS_DEF_STRING(empty, "", 0, 0)
NJS_DEF_STRING(_262, "$262", 0, 0)
NJS_DEF_STRING(_Infinity, "-Infinity", 0, 0)
NJS_DEF_STRING(AggregateError, "AggregateError", 0, 0)
NJS_DEF_STRING(All_promises_were_rejected, "All promises were rejected", 0, 0)
NJS_DEF_STRING(Array, "Array", 0, 0)
NJS_DEF_STRING(Array_Iterator, "Array Iterator", 0, 0)
NJS_DEF_STRING(ArrayBuffer, "ArrayBuffer", 0, 0)
NJS_DEF_STRING(AsyncFunction, "AsyncFunction", 0, 0)
NJS_DEF_STRING(Boolean, "Boolean", 0, 0)
NJS_DEF_STRING(Buffer, "Buffer", 0, 0)
NJS_DEF_STRING(BYTES_PER_ELEMENT, "BYTES_PER_ELEMENT", 0, 0)
NJS_DEF_STRING(DataView, "DataView", 0, 0)
NJS_DEF_STRING(Date, "Date", 0, 0)
NJS_DEF_STRING(E, "E", 0, 0)
NJS_DEF_STRING(EPSILON, "EPSILON", 0, 0)
NJS_DEF_STRING(Error, "Error", 0, 0)
NJS_DEF_STRING(EvalError, "EvalError", 0, 0)
NJS_DEF_STRING(Float32Array, "Float32Array", 0, 0)
NJS_DEF_STRING(Float64Array, "Float64Array", 0, 0)
NJS_DEF_STRING(Function, "Function", 0, 0)
NJS_DEF_STRING(Infinity, "Infinity", 0, 0)
NJS_DEF_STRING(Int16Array, "Int16Array", 0, 0)
NJS_DEF_STRING(Int32Array, "Int32Array", 0, 0)
NJS_DEF_STRING(Int8Array, "Int8Array", 0, 0)
NJS_DEF_STRING(InternalError, "InternalError", 0, 0)
NJS_DEF_STRING(Invalid_Date, "Invalid Date", 0, 0)
NJS_DEF_STRING(JSON, "JSON", 0, 0)
NJS_DEF_STRING(LN10, "LN10", 0, 0)
NJS_DEF_STRING(LN2, "LN2", 0, 0)
NJS_DEF_STRING(LOG10E, "LOG10E", 0, 0)
NJS_DEF_STRING(LOG2E, "LOG2E", 0, 0)
NJS_DEF_STRING(MAX_LENGTH, "MAX_LENGTH", 0, 0)
NJS_DEF_STRING(MAX_SAFE_INTEGER, "MAX_SAFE_INTEGER", 0, 0)
NJS_DEF_STRING(MAX_STRING_LENGTH, "MAX_STRING_LENGTH", 0, 0)
NJS_DEF_STRING(MAX_VALUE, "MAX_VALUE", 0, 0)
NJS_DEF_STRING(MIN_SAFE_INTEGER, "MIN_SAFE_INTEGER", 0, 0)
NJS_DEF_STRING(MIN_VALUE, "MIN_VALUE", 0, 0)
NJS_DEF_STRING(Math, "Math", 0, 0)
NJS_DEF_STRING(MemoryError, "MemoryError", 0, 0)
NJS_DEF_STRING(NEGATIVE_INFINITY, "NEGATIVE_INFINITY", 0, 0)
NJS_DEF_STRING(NaN, "NaN", 0, 0)
NJS_DEF_STRING(Number, "Number", 0, 0)
NJS_DEF_STRING(Object, "Object", 0, 0)
NJS_DEF_STRING(PI, "PI", 0, 0)
NJS_DEF_STRING(POSITIVE_INFINITY, "POSITIVE_INFINITY", 0, 0)
NJS_DEF_STRING(Promise, "Promise", 0, 0)
NJS_DEF_STRING(RangeError, "RangeError", 0, 0)
NJS_DEF_STRING(ReferenceError, "ReferenceError", 0, 0)
NJS_DEF_STRING(RegExp, "RegExp", 0, 0)
NJS_DEF_STRING(SQRT1_2, "SQRT1_2", 0, 0)
NJS_DEF_STRING(SQRT2, "SQRT2", 0, 0)
NJS_DEF_STRING(String, "String", 0, 0)
NJS_DEF_STRING(Symbol, "Symbol", 0, 0)
NJS_DEF_STRING(SyntaxError, "SyntaxError", 0, 0)
NJS_DEF_STRING(TextDecoder, "TextDecoder", 0, 0)
NJS_DEF_STRING(TextEncoder, "TextEncoder", 0, 0)
NJS_DEF_STRING(TypeError, "TypeError", 0, 0)
NJS_DEF_STRING(TypedArray, "TypedArray", 0, 0)
NJS_DEF_STRING(URIError, "URIError", 0, 0)
NJS_DEF_STRING(UTC, "UTC", 0, 0)
NJS_DEF_STRING(Uint16Array, "Uint16Array", 0, 0)
NJS_DEF_STRING(Uint32Array, "Uint32Array", 0, 0)
NJS_DEF_STRING(Uint8Array, "Uint8Array", 0, 0)
NJS_DEF_STRING(Uint8ClampedArray, "Uint8ClampedArray", 0, 0)
NJS_DEF_STRING(_Getter_, "[Getter]", 0, 0)
NJS_DEF_STRING(_Setter_, "[Setter]", 0, 0)
NJS_DEF_STRING(_Getter_Setter_, "[Getter/Setter]", 0, 0)
NJS_DEF_STRING(_object_Array_, "[object Array]", 0, 0)
NJS_DEF_STRING(_object_Arguments_, "[object Arguments]", 0, 0)
NJS_DEF_STRING(_object_Boolean_, "[object Boolean]", 0, 0)
NJS_DEF_STRING(_object_Date_, "[object Date]", 0, 0)
NJS_DEF_STRING(_object_Error_, "[object Error]", 0, 0)
NJS_DEF_STRING(_object_Function_, "[object Function]", 0, 0)
NJS_DEF_STRING(_object_Null_, "[object Null]", 0, 0)
NJS_DEF_STRING(_object_Number_, "[object Number]", 0, 0)
NJS_DEF_STRING(_object_Object_, "[object Object]", 0, 0)
NJS_DEF_STRING(_object_RegExp_, "[object RegExp]", 0, 0)
NJS_DEF_STRING(_object_String_, "[object String]", 0, 0)
NJS_DEF_STRING(_object_Undefined_, "[object Undefined]", 0, 0)
NJS_DEF_STRING(__proto__, "__proto__", 0, 0)
NJS_DEF_STRING(abs, "abs", 0, 0)
NJS_DEF_STRING(acos, "acos", 0, 0)
NJS_DEF_STRING(acosh, "acosh", 0, 0)
NJS_DEF_STRING(all, "all", 0, 0)
NJS_DEF_STRING(alloc, "alloc", 0, 0)
NJS_DEF_STRING(allocUnsafe, "allocUnsafe", 0, 0)
NJS_DEF_STRING(allocUnsafeSlow, "allocUnsafeSlow", 0, 0)
NJS_DEF_STRING(allSettled, "allSettled", 0, 0)
NJS_DEF_STRING(anonymous, "anonymous", 0, 0)
NJS_DEF_STRING(any, "any", 0, 0)
NJS_DEF_STRING(apply, "apply", 0, 0)
NJS_DEF_STRING(argv, "argv", 0, 0)
NJS_DEF_STRING(asin, "asin", 0, 0)
NJS_DEF_STRING(asinh, "asinh", 0, 0)
NJS_DEF_STRING(assign, "assign", 0, 0)
NJS_DEF_STRING(asyncIterator, "asyncIterator", 0, 0)
NJS_DEF_STRING(atan, "atan", 0, 0)
NJS_DEF_STRING(atan2, "atan2", 0, 0)
NJS_DEF_STRING(atanh, "atanh", 0, 0)
NJS_DEF_STRING(atob, "atob", 0, 0)
NJS_DEF_STRING(bind, "bind", 0, 0)
NJS_DEF_STRING(boolean, "boolean", 0, 0)
NJS_DEF_STRING(btoa, "btoa", 0, 0)
NJS_DEF_STRING(buffer, "buffer", 0, 0)
NJS_DEF_STRING(byteLength, "byteLength", 0, 0)
NJS_DEF_STRING(byteOffset, "byteOffset", 0, 0)
NJS_DEF_STRING(call, "call", 0, 0)
NJS_DEF_STRING(callee, "callee", 0, 0)
NJS_DEF_STRING(caller, "caller", 0, 0)
NJS_DEF_STRING(cbrt, "cbrt", 0, 0)
NJS_DEF_STRING(ceil, "ceil", 0, 0)
NJS_DEF_STRING(charAt, "charAt", 0, 0)
NJS_DEF_STRING(charCodeAt, "charCodeAt", 0, 0)
NJS_DEF_STRING(cluster_size, "cluster_size", 0, 0)
NJS_DEF_STRING(clz32, "clz32", 0, 0)
NJS_DEF_STRING(codePointAt, "codePointAt", 0, 0)
NJS_DEF_STRING(compare, "compare", 0, 0)
NJS_DEF_STRING(concat, "concat", 0, 0)
NJS_DEF_STRING(configurable, "configurable", 0, 0)
NJS_DEF_STRING(constructor, "constructor", 0, 0)
NJS_DEF_STRING(copy, "copy", 0, 0)
NJS_DEF_STRING(copyWithin, "copyWithin", 0, 0)
NJS_DEF_STRING(cos, "cos", 0, 0)
NJS_DEF_STRING(cosh, "cosh", 0, 0)
NJS_DEF_STRING(create, "create", 0, 0)
NJS_DEF_STRING(data, "data", 0, 0)
NJS_DEF_STRING(decode, "decode", 0, 0)
NJS_DEF_STRING(decodeURI, "decodeURI", 0, 0)
NJS_DEF_STRING(decodeURIComponent, "decodeURIComponent", 0, 0)
NJS_DEF_STRING(defineProperty, "defineProperty", 0, 0)
NJS_DEF_STRING(defineProperties, "defineProperties", 0, 0)
NJS_DEF_STRING(description, "description", 0, 0)
NJS_DEF_STRING(done, "done", 0, 0)
NJS_DEF_STRING(dump, "dump", 0, 0)
NJS_DEF_STRING(encode, "encode", 0, 0)
NJS_DEF_STRING(encodeInto, "encodeInto", 0, 0)
NJS_DEF_STRING(encodeURI, "encodeURI", 0, 0)
NJS_DEF_STRING(encodeURIComponent, "encodeURIComponent", 0, 0)
NJS_DEF_STRING(encoding, "encoding", 0, 0)
NJS_DEF_STRING(endsWith, "endsWith", 0, 0)
NJS_DEF_STRING(engine, "engine", 0, 0)
NJS_DEF_STRING(enumerable, "enumerable", 0, 0)
NJS_DEF_STRING(entries, "entries", 0, 0)
NJS_DEF_STRING(env, "env", 0, 0)
NJS_DEF_STRING(equals, "equals", 0, 0)
NJS_DEF_STRING(errors, "errors", 0, 0)
NJS_DEF_STRING(every, "every", 0, 0)
NJS_DEF_STRING(exec, "exec", 0, 0)
NJS_DEF_STRING(exp, "exp", 0, 0)
NJS_DEF_STRING(expm1, "expm1", 0, 0)
NJS_DEF_STRING(external, "external", 0, 0)
NJS_DEF_STRING(fatal, "fatal", 0, 0)
NJS_DEF_STRING(fileName, "fileName", 0, 0)
NJS_DEF_STRING(fill, "fill", 0, 0)
NJS_DEF_STRING(filter, "filter", 0, 0)
NJS_DEF_STRING(find, "find", 0, 0)
NJS_DEF_STRING(findIndex, "findIndex", 0, 0)
NJS_DEF_STRING(flags, "flags", 0, 0)
NJS_DEF_STRING(floor, "floor", 0, 0)
NJS_DEF_STRING(forEach, "forEach", 0, 0)
NJS_DEF_STRING(freeze, "freeze", 0, 0)
NJS_DEF_STRING(fromCharCode, "fromCharCode", 0, 0)
NJS_DEF_STRING(fromCodePoint, "fromCodePoint", 0, 0)
NJS_DEF_STRING(fround, "fround", 0, 0)
NJS_DEF_STRING(fulfilled, "fulfilled", 0, 0)
NJS_DEF_STRING(get, "get", 0, 0)
NJS_DEF_STRING(getDate, "getDate", 0, 0)
NJS_DEF_STRING(getDay, "getDay", 0, 0)
NJS_DEF_STRING(getFloat32, "getFloat32", 0, 0)
NJS_DEF_STRING(getFloat64, "getFloat64", 0, 0)
NJS_DEF_STRING(getFullYear, "getFullYear", 0, 0)
NJS_DEF_STRING(getHours, "getHours", 0, 0)
NJS_DEF_STRING(getInt16, "getInt16", 0, 0)
NJS_DEF_STRING(getInt32, "getInt32", 0, 0)
NJS_DEF_STRING(getInt8, "getInt8", 0, 0)
NJS_DEF_STRING(getMilliseconds, "getMilliseconds", 0, 0)
NJS_DEF_STRING(getMinutes, "getMinutes", 0, 0)
NJS_DEF_STRING(getOwnPropertyDescriptor, "getOwnPropertyDescriptor", 0, 0)
NJS_DEF_STRING(getOwnPropertyDescriptors, "getOwnPropertyDescriptors", 0, 0)
NJS_DEF_STRING(getOwnPropertyNames, "getOwnPropertyNames", 0, 0)
NJS_DEF_STRING(getOwnPropertySymbols, "getOwnPropertySymbols", 0, 0)
NJS_DEF_STRING(getPrototypeOf, "getPrototypeOf", 0, 0)
NJS_DEF_STRING(getSeconds, "getSeconds", 0, 0)
NJS_DEF_STRING(getTime, "getTime", 0, 0)
NJS_DEF_STRING(getTimezoneOffset, "getTimezoneOffset", 0, 0)
NJS_DEF_STRING(getUTCDate, "getUTCDate", 0, 0)
NJS_DEF_STRING(getUTCDay, "getUTCDay", 0, 0)
NJS_DEF_STRING(getUTCFullYear, "getUTCFullYear", 0, 0)
NJS_DEF_STRING(getUTCHours, "getUTCHours", 0, 0)
NJS_DEF_STRING(getUTCMilliseconds, "getUTCMilliseconds", 0, 0)
NJS_DEF_STRING(getUTCMinutes, "getUTCMinutes", 0, 0)
NJS_DEF_STRING(getUTCMonth, "getUTCMonth", 0, 0)
NJS_DEF_STRING(getUTCSeconds, "getUTCSeconds", 0, 0)
NJS_DEF_STRING(getUint16, "getUint16", 0, 0)
NJS_DEF_STRING(getUint32, "getUint32", 0, 0)
NJS_DEF_STRING(getUint8, "getUint8", 0, 0)
NJS_DEF_STRING(getMonth, "getMonth", 0, 0)
NJS_DEF_STRING(global, "global", 0, 0)
NJS_DEF_STRING(globalThis, "globalThis", 0, 0)
NJS_DEF_STRING(groups, "groups", 0, 0)
NJS_DEF_STRING(hasOwnProperty, "hasOwnProperty", 0, 0)
NJS_DEF_STRING(hasInstance, "hasInstance", 0, 0)
NJS_DEF_STRING(hypot, "hypot", 0, 0)
NJS_DEF_STRING(ignoreBOM, "ignoreBOM", 0, 0)
NJS_DEF_STRING(ignoreCase, "ignoreCase", 0, 0)
NJS_DEF_STRING(imul, "imul", 0, 0)
NJS_DEF_STRING(includes, "includes", 0, 0)
NJS_DEF_STRING(index, "index", 0, 0)
NJS_DEF_STRING(indexOf, "indexOf", 0, 0)
NJS_DEF_STRING(input, "input", 0, 0)
NJS_DEF_STRING(is, "is", 0, 0)
NJS_DEF_STRING(isArray, "isArray", 0, 0)
NJS_DEF_STRING(isBuffer, "isBuffer", 0, 0)
NJS_DEF_STRING(isConcatSpreadable, "isConcatSpreadable", 0, 0)
NJS_DEF_STRING(isEncoding, "isEncoding", 0, 0)
NJS_DEF_STRING(isExtensible, "isExtensible", 0, 0)
NJS_DEF_STRING(isFinite, "isFinite", 0, 0)
NJS_DEF_STRING(isFrozen, "isFrozen", 0, 0)
NJS_DEF_STRING(isInteger, "isInteger", 0, 0)
NJS_DEF_STRING(isNaN, "isNaN", 0, 0)
NJS_DEF_STRING(isPrototypeOf, "isPrototypeOf", 0, 0)
NJS_DEF_STRING(isSafeInteger, "isSafeInteger", 0, 0)
NJS_DEF_STRING(isSealed, "isSealed", 0, 0)
NJS_DEF_STRING(isView, "isView", 0, 0)
NJS_DEF_STRING(iterator, "iterator", 0, 0)
NJS_DEF_STRING(join, "join", 0, 0)
NJS_DEF_STRING(keyFor, "keyFor", 0, 0)
NJS_DEF_STRING(keys, "keys", 0, 0)
NJS_DEF_STRING(kill, "kill", 0, 0)
NJS_DEF_STRING(lastIndex, "lastIndex", 0, 0)
NJS_DEF_STRING(lastIndexOf, "lastIndexOf", 0, 0)
NJS_DEF_STRING(length, "length", 0, 0)
NJS_DEF_STRING(lineNumber, "lineNumber", 0, 0)
NJS_DEF_STRING(log, "log", 0, 0)
NJS_DEF_STRING(log10, "log10", 0, 0)
NJS_DEF_STRING(log1p, "log1p", 0, 0)
NJS_DEF_STRING(log2, "log2", 0, 0)
NJS_DEF_STRING(map, "map", 0, 0)
NJS_DEF_STRING(matchAll, "matchAll", 0, 0)
NJS_DEF_STRING(match, "match", 0, 0)
NJS_DEF_STRING(max, "max", 0, 0)
NJS_DEF_STRING(min, "min", 0, 0)
NJS_DEF_STRING(memoryStats, "memoryStats", 0, 0)
NJS_DEF_STRING(message, "message", 0, 0)
NJS_DEF_STRING(multiline, "multiline", 0, 0)
NJS_DEF_STRING(name, "name", 0, 0)
NJS_DEF_STRING(nblocks, "nblocks", 0, 0)
NJS_DEF_STRING(next, "next", 0, 0)
NJS_DEF_STRING(njs, "njs", 0, 0)
NJS_DEF_STRING(now, "now", 0, 0)
NJS_DEF_STRING(number, "number", 0, 0)
NJS_DEF_STRING(object, "object", 0, 0)
NJS_DEF_STRING(on, "on", 0, 0)
NJS_DEF_STRING(padEnd, "padEnd", 0, 0)
NJS_DEF_STRING(padStart, "padStart", 0, 0)
NJS_DEF_STRING(page_size, "page_size", 0, 0)
NJS_DEF_STRING(parse, "parse", 0, 0)
NJS_DEF_STRING(parseFloat, "parseFloat", 0, 0)
NJS_DEF_STRING(parseInt, "parseInt", 0, 0)
NJS_DEF_STRING(pid, "pid", 0, 0)
NJS_DEF_STRING(pop, "pop", 0, 0)
NJS_DEF_STRING(pow, "pow", 0, 0)
NJS_DEF_STRING(ppid, "ppid", 0, 0)
NJS_DEF_STRING(preventExtensions, "preventExtensions", 0, 0)
NJS_DEF_STRING(process, "process", 0, 0)
NJS_DEF_STRING(propertyIsEnumerable, "propertyIsEnumerable", 0, 0)
NJS_DEF_STRING(prototype, "prototype", 0, 0)
NJS_DEF_STRING(push, "push", 0, 0)
NJS_DEF_STRING(race, "race", 0, 0)
NJS_DEF_STRING(random, "random", 0, 0)
NJS_DEF_STRING(read, "read", 0, 0)
NJS_DEF_STRING(readDoubleBE, "readDoubleBE", 0, 0)
NJS_DEF_STRING(readDoubleLE, "readDoubleLE", 0, 0)
NJS_DEF_STRING(readFloatBE, "readFloatBE", 0, 0)
NJS_DEF_STRING(readFloatLE, "readFloatLE", 0, 0)
NJS_DEF_STRING(readInt16BE, "readInt16BE", 0, 0)
NJS_DEF_STRING(readInt16LE, "readInt16LE", 0, 0)
NJS_DEF_STRING(readInt32BE, "readInt32BE", 0, 0)
NJS_DEF_STRING(readInt32LE, "readInt32LE", 0, 0)
NJS_DEF_STRING(readInt8, "readInt8", 0, 0)
NJS_DEF_STRING(readIntBE, "readIntBE", 0, 0)
NJS_DEF_STRING(readIntLE, "readIntLE", 0, 0)
NJS_DEF_STRING(readUInt16BE, "readUInt16BE", 0, 0)
NJS_DEF_STRING(readUInt16LE, "readUInt16LE", 0, 0)
NJS_DEF_STRING(readUInt32BE, "readUInt32BE", 0, 0)
NJS_DEF_STRING(readUInt32LE, "readUInt32LE", 0, 0)
NJS_DEF_STRING(readUInt8, "readUInt8", 0, 0)
NJS_DEF_STRING(readUIntBE, "readUIntBE", 0, 0)
NJS_DEF_STRING(readUIntLE, "readUIntLE", 0, 0)
NJS_DEF_STRING(reason, "reason", 0, 0)
NJS_DEF_STRING(reduce, "reduce", 0, 0)
NJS_DEF_STRING(reduceRight, "reduceRight", 0, 0)
NJS_DEF_STRING(reject, "reject", 0, 0)
NJS_DEF_STRING(rejected, "rejected", 0, 0)
NJS_DEF_STRING(repeat, "repeat", 0, 0)
NJS_DEF_STRING(replace, "replace", 0, 0)
NJS_DEF_STRING(replaceAll, "replaceAll", 0, 0)
NJS_DEF_STRING(require, "require", 0, 0)
NJS_DEF_STRING(resolve, "resolve", 0, 0)
NJS_DEF_STRING(reverse, "reverse", 0, 0)
NJS_DEF_STRING(round, "round", 0, 0)
NJS_DEF_STRING(seal, "seal", 0, 0)
NJS_DEF_STRING(search, "search", 0, 0)
NJS_DEF_STRING(set, "set", 0, 0)
NJS_DEF_STRING(setDate, "setDate", 0, 0)
NJS_DEF_STRING(setFloat32, "setFloat32", 0, 0)
NJS_DEF_STRING(setFloat64, "setFloat64", 0, 0)
NJS_DEF_STRING(setFullYear, "setFullYear", 0, 0)
NJS_DEF_STRING(setHours, "setHours", 0, 0)
NJS_DEF_STRING(setInt16, "setInt16", 0, 0)
NJS_DEF_STRING(setInt32, "setInt32", 0, 0)
NJS_DEF_STRING(setInt8, "setInt8", 0, 0)
NJS_DEF_STRING(setMilliseconds, "setMilliseconds", 0, 0)
NJS_DEF_STRING(setMinutes, "setMinutes", 0, 0)
NJS_DEF_STRING(setMonth, "setMonth", 0, 0)
NJS_DEF_STRING(setPrototypeOf, "setPrototypeOf", 0, 0)
NJS_DEF_STRING(setSeconds, "setSeconds", 0, 0)
NJS_DEF_STRING(setTime, "setTime", 0, 0)
NJS_DEF_STRING(setUint16, "setUint16", 0, 0)
NJS_DEF_STRING(setUint32, "setUint32", 0, 0)
NJS_DEF_STRING(setUint8, "setUint8", 0, 0)
NJS_DEF_STRING(setUTCDate, "setUTCDate", 0, 0)
NJS_DEF_STRING(setUTCFullYear, "setUTCFullYear", 0, 0)
NJS_DEF_STRING(setUTCHours, "setUTCHours", 0, 0)
NJS_DEF_STRING(setUTCMilliseconds, "setUTCMilliseconds", 0, 0)
NJS_DEF_STRING(setUTCMinutes, "setUTCMinutes", 0, 0)
NJS_DEF_STRING(setUTCMonth, "setUTCMonth", 0, 0)
NJS_DEF_STRING(setUTCSeconds, "setUTCSeconds", 0, 0)
NJS_DEF_STRING(shift, "shift", 0, 0)
NJS_DEF_STRING(sign, "sign", 0, 0)
NJS_DEF_STRING(sin, "sin", 0, 0)
NJS_DEF_STRING(sinh, "sinh", 0, 0)
NJS_DEF_STRING(size, "size", 0, 0)
NJS_DEF_STRING(slice, "slice", 0, 0)
NJS_DEF_STRING(some, "some", 0, 0)
NJS_DEF_STRING(sort, "sort", 0, 0)
NJS_DEF_STRING(source, "source", 0, 0)
NJS_DEF_STRING(species, "species", 0, 0)
NJS_DEF_STRING(splice, "splice", 0, 0)
NJS_DEF_STRING(split, "split", 0, 0)
NJS_DEF_STRING(sqrt, "sqrt", 0, 0)
NJS_DEF_STRING(stack, "stack", 0, 0)
NJS_DEF_STRING(startsWith, "startsWith", 0, 0)
NJS_DEF_STRING(status, "status", 0, 0)
NJS_DEF_STRING(sticky, "sticky", 0, 0)
NJS_DEF_STRING(stream, "stream", 0, 0)
NJS_DEF_STRING(string, "string", 0, 0)
NJS_DEF_STRING(stringify, "stringify", 0, 0)
NJS_DEF_STRING(subarray, "subarray", 0, 0)
NJS_DEF_STRING(substr, "substr", 0, 0)
NJS_DEF_STRING(substring, "substring", 0, 0)
NJS_DEF_STRING(symbol, "symbol", 0, 0)
NJS_DEF_STRING(swap16, "swap16", 0, 0)
NJS_DEF_STRING(swap32, "swap32", 0, 0)
NJS_DEF_STRING(swap64, "swap64", 0, 0)
NJS_DEF_STRING(tan, "tan", 0, 0)
NJS_DEF_STRING(tanh, "tanh", 0, 0)
NJS_DEF_STRING(test, "test", 0, 0)
NJS_DEF_STRING(then, "then", 0, 0)
NJS_DEF_STRING(times, "times", 0, 0)
NJS_DEF_STRING(toDateString, "toDateString", 0, 0)
NJS_DEF_STRING(toExponential, "toExponential", 0, 0)
NJS_DEF_STRING(toFixed, "toFixed", 0, 0)
NJS_DEF_STRING(toISOString, "toISOString", 0, 0)
NJS_DEF_STRING(toJSON, "toJSON", 0, 0)
NJS_DEF_STRING(toLocaleDateString, "toLocaleDateString", 0, 0)
NJS_DEF_STRING(toLocaleString, "toLocaleString", 0, 0)
NJS_DEF_STRING(toLocaleTimeString, "toLocaleTimeString", 0, 0)
NJS_DEF_STRING(toLowerCase, "toLowerCase", 0, 0)
NJS_DEF_STRING(toPrimitive, "toPrimitive", 0, 0)
NJS_DEF_STRING(toPrecision, "toPrecision", 0, 0)
NJS_DEF_STRING(toReversed, "toReversed", 0, 0)
NJS_DEF_STRING(toSorted, "toSorted", 0, 0)
NJS_DEF_STRING(toSpliced, "toSpliced", 0, 0)
NJS_DEF_STRING(toStringTag, "toStringTag", 0, 0)
NJS_DEF_STRING(toString, "toString", 0, 0)
NJS_DEF_STRING(toTimeString, "toTimeString", 0, 0)
NJS_DEF_STRING(toUpperCase, "toUpperCase", 0, 0)
NJS_DEF_STRING(toUTCString, "toUTCString", 0, 0)
NJS_DEF_STRING(trimEnd, "trimEnd", 0, 0)
NJS_DEF_STRING(trimStart, "trimStart", 0, 0)
NJS_DEF_STRING(trim, "trim", 0, 0)
NJS_DEF_STRING(trunc, "trunc", 0, 0)
NJS_DEF_STRING(type, "type", 0, 0)
NJS_DEF_STRING(usec, "usec", 0, 0)
NJS_DEF_STRING(unscopables, "unscopables", 0, 0)
NJS_DEF_STRING(unshift, "unshift", 0, 0)
NJS_DEF_STRING(utf_8, "utf-8", 0, 0)
NJS_DEF_STRING(value, "value", 0, 0)
NJS_DEF_STRING(valueOf, "valueOf", 0, 0)
NJS_DEF_STRING(values, "values", 0, 0)
NJS_DEF_STRING(version, "version", 0, 0)
NJS_DEF_STRING(version_number, "version_number", 0, 0)
NJS_DEF_STRING(write, "write", 0, 0)
NJS_DEF_STRING(writable, "writable", 0, 0)
NJS_DEF_STRING(writeDoubleBE, "writeDoubleBE", 0, 0)
NJS_DEF_STRING(writeDoubleLE, "writeDoubleLE", 0, 0)
NJS_DEF_STRING(writeFloatBE, "writeFloatBE", 0, 0)
NJS_DEF_STRING(writeFloatLE, "writeFloatLE", 0, 0)
NJS_DEF_STRING(writeIntBE, "writeIntBE", 0, 0)
NJS_DEF_STRING(writeIntLE, "writeIntLE", 0, 0)
NJS_DEF_STRING(writeInt16BE, "writeInt16BE", 0, 0)
NJS_DEF_STRING(writeInt16LE, "writeInt16LE", 0, 0)
NJS_DEF_STRING(writeInt32BE, "writeInt32BE", 0, 0)
NJS_DEF_STRING(writeInt32LE, "writeInt32LE", 0, 0)
NJS_DEF_STRING(writeInt8, "writeInt8", 0, 0)
NJS_DEF_STRING(writeUIntBE, "writeUIntBE", 0, 0)
NJS_DEF_STRING(writeUIntLE, "writeUIntLE", 0, 0)
NJS_DEF_STRING(writeUInt16BE, "writeUInt16BE", 0, 0)
NJS_DEF_STRING(writeUInt16LE, "writeUInt16LE", 0, 0)
NJS_DEF_STRING(writeUInt32BE, "writeUInt32BE", 0, 0)
NJS_DEF_STRING(writeUInt32LE, "writeUInt32LE", 0, 0)
NJS_DEF_STRING(writeUInt8, "writeUInt8", 0, 0)
NJS_DEF_STRING(written, "written", 0, 0)
