
/* This file is auto-generated by configure */


#ifndef NJS_CLANG
#define NJS_CLANG  1
#endif


#ifndef NJS_INT_SIZE
#define NJS_INT_SIZE  4
#endif


#ifndef NJS_UINT_SIZE
#define NJS_UINT_SIZE  4
#endif


#ifndef NJS_PTR_SIZE
#define NJS_PTR_SIZE  8
#endif


#ifndef NJS_UINTPTR_T_SIZE
#define NJS_UINTPTR_T_SIZE  8
#endif


#ifndef NJS_SIZE_T_SIZE
#define NJS_SIZE_T_SIZE  8
#endif


#ifndef NJS_OFF_T_SIZE
#define NJS_OFF_T_SIZE  8
#endif


#ifndef NJS_TIME_T_SIZE
#define NJS_TIME_T_SIZE  8
#endif


#ifndef NJS_HAVE_FP_EVAL_METHOD
#define NJS_HAVE_FP_EVAL_METHOD  1
#endif


#ifndef NJS_BYTE_ORDER
#define NJS_BYTE_ORDER  little
#endif


#ifndef NJS_HAVE_LITTLE_ENDIAN
#define NJS_HAVE_LITTLE_ENDIAN  1
#endif


#ifndef NJS_HAVE_UNSIGNED_INT128
#define NJS_HAVE_UNSIGNED_INT128  1
#endif


#ifndef NJS_HAVE_BUILTIN_EXPECT
#define NJS_HAVE_BUILTIN_EXPECT  1
#endif


#ifndef NJS_HAVE_BUILTIN_UNREACHABLE
#define NJS_HAVE_BUILTIN_UNREACHABLE  1
#endif


#ifndef NJS_HAVE_BUILTIN_PREFETCH
#define NJS_HAVE_BUILTIN_PREFETCH  1
#endif


#ifndef NJS_HAVE_BUILTIN_CLZ
#define NJS_HAVE_BUILTIN_CLZ  1
#endif


#ifndef NJS_HAVE_BUILTIN_CLZLL
#define NJS_HAVE_BUILTIN_CLZLL  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_VISIBILITY
#define NJS_HAVE_GCC_ATTRIBUTE_VISIBILITY  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_MALLOC
#define NJS_HAVE_GCC_ATTRIBUTE_MALLOC  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_ALIGNED
#define NJS_HAVE_GCC_ATTRIBUTE_ALIGNED  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_PACKED
#define NJS_HAVE_GCC_ATTRIBUTE_PACKED  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_FALLTHROUGH
#define NJS_HAVE_GCC_ATTRIBUTE_FALLTHROUGH  1
#endif


#ifndef NJS_HAVE_GCC_ATTRIBUTE_NO_SANITIZE
#define NJS_HAVE_GCC_ATTRIBUTE_NO_SANITIZE  1
#endif


#ifndef NJS_HAVE_CLOCK_MONOTONIC
#define NJS_HAVE_CLOCK_MONOTONIC  1
#endif


#ifndef NJS_HAVE_TM_GMTOFF
#define NJS_HAVE_TM_GMTOFF  1
#endif


#ifndef NJS_HAVE_POSIX_MEMALIGN
#define NJS_HAVE_POSIX_MEMALIGN  1
#endif


#ifndef NJS_HAVE_GETENTROPY
#define NJS_HAVE_GETENTROPY  1
#endif


#ifndef NJS_HAVE_STAT_ATIMESPEC
#define NJS_HAVE_STAT_ATIMESPEC  1
#endif


#ifndef NJS_HAVE__STAT_BIRTHTIM
#define NJS_HAVE__STAT_BIRTHTIM  1
#endif


#ifndef NJS_HAVE_STAT_ATIM
#define NJS_HAVE_STAT_ATIM  1
#endif


#ifndef NJS_HAVE_COMPUTED_GOTO
#define NJS_HAVE_COMPUTED_GOTO  1
#endif


#ifndef NJS_HAVE_EXPLICIT_BZERO
#define NJS_HAVE_EXPLICIT_BZERO  1
#endif


#ifndef NJS_HAVE_PCRE2
#define NJS_HAVE_PCRE2  1
#endif


#ifndef NJS_PCRE2_VERSION
#define NJS_PCRE2_VERSION  10.44
#endif


#ifndef NJS_HAVE_OPENSSL
#define NJS_HAVE_OPENSSL  1
#endif


#ifndef NJS_OPENSSL_VERSION
#define NJS_OPENSSL_VERSION  "LibreSSL 4.3.0"
#endif


#ifndef NJS_HAVE_LIBXML2
#define NJS_HAVE_LIBXML2  1
#endif


#ifndef NJS_LIBXML2_VERSION
#define NJS_LIBXML2_VERSION  "2.15.3"
#endif


#ifndef NJS_HAVE_ZLIB
#define NJS_HAVE_ZLIB  1
#endif


#ifndef NJS_ZLIB_VERSION
#define NJS_ZLIB_VERSION  "1.3.2"
#endif

